@@ -312,3 +312,13 @@ setClass("cutoff", representation = representation(name = "character",
                    prototype = prototype(name = "empirical",
                                          fct = function(data) quantile(data),
                                          cutoff.quantile = 0.95))
+
+
+#################################################
+# new risk classes
+#################################################
+setClass("interpolRisk", representation = representation(samplesize="numeric"),
+                         contains = c("VIRTUAL", "RiskType"))
+setClass("OMSRRisk", contains = "interpolRisk", prototype=prototype(type=".OMSE", samplesize=100))
+setClass("RMXRRisk", contains = "interpolRisk", prototype=prototype(type=".RMXE", samplesize=100))
+setClass("MBRRisk", contains = "interpolRisk", prototype=prototype(type=".MBRE",samplesize=100))
