@@ -4,4 +4,6 @@ MBRRisk <- function(samplesize = 100)  new("MBRRisk",type=".MBRE", samplesize =
 RMXRRisk <- function(samplesize = 100)  new("RMXRRisk",type=".RMXE", samplesize = samplesize)
 setMethod("samplesize","interpolRisk", function(object)object@samplesize)
 setReplaceMethod("samplesize","interpolRisk", function(object, value){
-       object@samplesize <- value; object})
\ No newline at end of file
+       object@samplesize <- value; object})
+setMethod("biastype","interpolRisk", function(object) symmetricBias())
+setMethod("normtype","interpolRisk", function(object) NormType())
