@@ -103,7 +103,22 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
 
         ### update - function
         updateStep <- function(u.theta, theta, IC, L2Fam, Param,
-                               withModif = TRUE, with.u.var = FALSE){
+                               withPreModif = FALSE,
+                               withPostModif = TRUE, with.u.var = FALSE
+                               ){
+
+                if(withPreModif){
+                   main(Param)[] <- .deleteDim(u.theta[idx])
+#                   print(Param)
+                   if (lnx) nuisance(Param)[] <- .deleteDim(u.theta[nuis.idx])
+#                   print(Param)
+#                   print(L2Fam)
+                   L2Fam <- modifyModel(L2Fam, Param,
+                               .withL2derivDistr = L2Fam@.withEvalL2derivDistr)
+#                   print(L2Fam)
+                   IC <- modifyIC(IC)(L2Fam, IC)
+ #                  print(IC)
+                }
 
                 IC.c <- as(diag(p) %*% IC@Curve, "EuclRandVariable")
 
@@ -113,6 +128,7 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
                 IC.tot.0 <- NULL
 #                print(Dtau)
                 if(!.isUnitMatrix(Dtau)){
+ #                    print("HU1!")
                      Dminus <- solve(Dtau, generalized = TRUE)
                      projker <- diag(k) - Dminus %*% Dtau
 
@@ -146,6 +162,7 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
 
                      theta <- (tf$fct(u.theta))$fval
                 }else{
+#                     print("HU2!")
                      correct <- rowMeans(evalRandVar(IC.c, x0), na.rm = na.rm )
                      iM <- is.matrix(theta)
                      names(correct) <- if(iM) rownames(theta) else names(theta)
@@ -160,6 +177,7 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
                      IC.tot <- IC.c
                      u.theta <- theta
                 }
+#                print("HU3!")
 
                 var0 <- u.var <- NULL
                 if(with.u.var){
@@ -180,8 +198,7 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
                       if(withEvalAsVar) var0 <- eval(var0)
                    }
                 }
-
-                if(withModif){
+                if(withPostModif){
                    main(Param)[] <- .deleteDim(u.theta[idx])
                    if (lnx) nuisance(Param)[] <- .deleteDim(u.theta[nuis.idx])
 #                   print(L2Fam)
@@ -219,9 +236,11 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
                   L2Fam <- upd$L2Fam
                   Param <- upd$Param
                   tf <- trafo(L2Fam, Param)
-               }
+                  withPre <- FALSE
+               }else withPre <- TRUE
                upd <- updateStep(u.theta,theta,IC, L2Fam, Param,
-                                 withModif = (steps>1) | useLast,
+                                 withPreModif = withPre,
+                                 withPostModif = (steps>i) | useLast,
                                  with.u.var = i==steps)
                uksteps[,i] <- u.theta <- upd$u.theta
                ksteps[,i] <- theta <- upd$theta
@@ -323,7 +342,7 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
           dimnames(asVar) <- list(nms.theta.idx, nms.theta.idx)
         }
 
-        return(new("kStepEstimate", estimate.call = es.call,
+        estres <- new("kStepEstimate", estimate.call = es.call,
                 name = paste(steps, "-step estimate", sep = ""),
                 estimate = theta, samplesize = nrow(x0), asvar = asVar,
                 trafo = tf, fixed = fixed, nuis.idx = nuis.idx,
@@ -331,7 +350,9 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
                 untransformed.asvar = u.var, asbias = asBias, pIC = IC,
                 steps = steps, Infos = Infos, start = start,
                 startval = start.val, ustartval = u.start.val, ksteps = ksteps,
-                uksteps = uksteps, pICList = pICList, ICList = ICList))
+                uksteps = uksteps, pICList = pICList, ICList = ICList)
+        return(.checkEstClassForParamFamily(L2Fam,estres))
+
 }
 #  (est1.NS <- kStepEstimator(x, IC2.NS, est0, steps = 1))
 
